/* Copyright 2019, Realtek Semiconductor corp. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdint.h>

#define USB_MAXINTERFACES 32
#define INTERFACE_DESC_LENGTHi 9
#define DESC_HEADER_LENGTH 2
#define INTERFACE_DESC_LENGTH 9
#define USB_MAXENDPOINTS 32
#define ENDPOINT_AUDIO_DESC_LENGTH 9
#define ENDPOINT_DESC_LENGTH 7
#define RTS_DT_INTERFACE_SIZE 9

#define SC_VIDEOCONTROL 0x01
#define VC_EXTENSION_UNIT 0x06

#define UVC_GUID_REALTEK_VENDOR_CMD                                            \
  {                                                                            \
    0x8c, 0xa7, 0x29, 0x12, 0xb4, 0x47, 0x94, 0x40, 0xb0, 0xce, 0xdb, 0x07,    \
        0x38, 0x6f, 0xb9, 0x38                                                 \
  }

enum RTS_ERRNO {
  RTS_OK = 0,
  RTS_E_NULL_POINT,
  RTS_E_NOT_FOUND,
  RTS_E_INVALID_ARG,
  RTS_E_NOT_OPEN,
  RTS_E_NO_MEMORY,
  RTS_E_IO,
};

struct rts_endpoint_descriptor {
  uint8_t bLength;

  uint8_t bDescriptorType;

  uint8_t bEndpointAddress;

  uint8_t bmAttributes;

  uint16_t wMaxPacketSize;

  uint8_t bInterval;

  uint8_t bRefresh;

  uint8_t bSynchAddress;

  const unsigned char *extra;

  int extra_length;
};

struct rts_interface_descriptor {
  uint8_t bLength;

  uint8_t bDescriptorType;

  uint8_t bInterfaceNumber;

  uint8_t bAlternateSetting;

  uint8_t bNumEndpoints;

  uint8_t bInterfaceClass;

  uint8_t bInterfaceSubClass;

  uint8_t bInterfaceProtocol;

  uint8_t iInterface;

  const struct rts_endpoint_descriptor *endpoint;

  const unsigned char *extra;

  int extra_length;
};

struct rts_interface {
  const struct rts_interface_descriptor *altsetting;

  int num_altsetting;
};

struct rts_config_descriptor {
  uint8_t bLength;

  uint8_t bDescriptorType;

  uint16_t wTotalLength;

  uint8_t bNumInterfaces;

  uint8_t bConfigurationValue;

  uint8_t iConfiguration;

  uint8_t bmAttributes;

  uint8_t MaxPower;

  const struct rts_interface *interface;

  const unsigned char *extra;

  int extra_length;
};

struct rts_usb_descriptor_header {
  uint8_t bLength;
  uint8_t bDescriptorType;
};

enum rts_descriptor_type {
  RTS_DT_DEVICE = 0x01,

  RTS_DT_CONFIG = 0x02,

  RTS_DT_STRING = 0x03,

  RTS_DT_INTERFACE = 0x04,

  RTS_DT_ENDPOINT = 0x05,

  RTS_DT_HID = 0x21,

  RTS_DT_REPORT = 0x22,

  RTS_DT_PHYSICAL = 0x23,

  RTS_DT_HUB = 0x29,
};

int rts_get_config_descriptor(int fd, uint8_t config_index,
                              struct rts_config_descriptor **config);
void rts_clear_configuration(struct rts_config_descriptor *config);
